@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -271,6 +271,8 @@ void D10State::init() {
 
 	ID3D10Texture2D* pBackBuffer = NULL;
 	hr = pSwapChain->GetBuffer(0, __uuidof(*pBackBuffer), (LPVOID*)&pBackBuffer);
+	if (FAILED(hr))
+		ods("D3D10: pSwapChain->GetBuffer failure!");
 
 	pDevice->ClearState();
 
@@ -287,6 +289,8 @@ void D10State::init() {
 	pDevice->RSSetViewports(1, &vp);
 
 	hr = pDevice->CreateRenderTargetView(pBackBuffer, NULL, &pRTV);
+	if (FAILED(hr))
+		ods("D3D10: pDevice->CreateRenderTargetView failure!");
 
 	pDevice->OMSetRenderTargets(1, &pRTV, NULL);
 
@@ -324,6 +328,8 @@ void D10State::init() {
 	D3D10_PASS_DESC PassDesc;
 	pTechnique->GetPassByIndex(0)->GetDesc(&PassDesc);
 	hr = pDevice->CreateInputLayout(layout, numElements, PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &pVertexLayout);
+	if (FAILED(hr))
+		ods("D3D10: pDevice->CreateInputLayout failure!");
 	pDevice->IASetInputLayout(pVertexLayout);
 
 	D3D10_BUFFER_DESC bd;
@@ -334,6 +340,8 @@ void D10State::init() {
 	bd.CPUAccessFlags = D3D10_CPU_ACCESS_WRITE;
 	bd.MiscFlags = 0;
 	hr = pDevice->CreateBuffer(&bd, NULL, &pVertexBuffer);
+	if (FAILED(hr))
+		ods("D3D10: pDevice->CreateBuffer failure!");
 
 	DWORD indices[] = {
 		0,1,3,
@@ -349,6 +357,8 @@ void D10State::init() {
 	ZeroMemory(&InitData, sizeof(InitData));
 	InitData.pSysMem = indices;
 	hr = pDevice->CreateBuffer(&bd, &InitData, &pIndexBuffer);
+	if (FAILED(hr))
+		ods("D3D10: pDevice->CreateBuffer failure!");
 
 	// Set index buffer
 	pDevice->IASetIndexBuffer(pIndexBuffer, DXGI_FORMAT_R32_UINT, 0);
@@ -446,8 +456,8 @@ static HRESULT __stdcall myPresent(IDXGISwapChain *pSwapChain, UINT SyncInterval
 		D10State *ds = chains[pSwapChain];
 		if (ds && ds->pDevice != pDevice) {
 			ods("DXGI: SwapChain device changed");
-			delete ds;
 			devices.erase(ds->pDevice);
+			delete ds;
 			ds = NULL;
 		}
 		if (! ds) {
@@ -541,8 +551,8 @@ static void HookResizeRaw(voidFunc vfResize) {
 
 void checkDXGIHook(bool preonly) {
 	if (bChaining) {
-		return;
 		ods("DXGI: Causing a chain");
+		return;
 	}
 
 	if (! dxgi->iOffsetPresent || ! dxgi->iOffsetResize)
@@ -587,6 +597,11 @@ void checkDXGIHook(bool preonly) {
 }
 
 extern "C" __declspec(dllexport) void __cdecl PrepareDXGI() {
+	// This function is called by the Mumble client in Mumble's scope
+	// mainly to extract the offsets of various functions in the IDXGISwapChain
+	// and IDXGIObject interfaces that need to be hooked in target
+	// applications. The data is stored in the dxgi shared memory structure.
+
 	if (! dxgi)
 		return;
 
@@ -603,7 +618,7 @@ extern "C" __declspec(dllexport) void __cdecl PrepareDXGI() {
 	memset(&ovi, 0, sizeof(ovi));
 	ovi.dwOSVersionInfoSize = sizeof(ovi);
 	GetVersionExW(reinterpret_cast<OSVERSIONINFOW *>(&ovi));
-	if ((ovi.dwMajorVersion >= 7) || ((ovi.dwMajorVersion == 6) && (ovi.dwBuildNumber >= 6001))) {
+	if ((ovi.dwMajorVersion >= 7) || ((ovi.dwMajorVersion == 6) && (ovi.dwBuildNumber >= 6001))) { // Make sure this is vista or greater as quite a number of <=WinXP users have fake DX10 libs installed
 		HMODULE hD3D10 = LoadLibrary("D3D10.DLL");
 		HMODULE hDXGI = LoadLibrary("DXGI.DLL");
 
@@ -611,9 +626,10 @@ extern "C" __declspec(dllexport) void __cdecl PrepareDXGI() {
 			CreateDXGIFactoryType pCreateDXGIFactory = reinterpret_cast<CreateDXGIFactoryType>(GetProcAddress(hDXGI, "CreateDXGIFactory"));
 			ods("Got %p", pCreateDXGIFactory);
 			if (pCreateDXGIFactory) {
-				HRESULT hr;
 				IDXGIFactory * pFactory;
-				hr = pCreateDXGIFactory(__uuidof(IDXGIFactory), (void**)(&pFactory));
+				HRESULT hr = pCreateDXGIFactory(__uuidof(IDXGIFactory), (void**)(&pFactory));
+				if (FAILED(hr))
+					ods("D3D10: pCreateDXGIFactory failure!");
 				if (pFactory) {
 					HWND hwnd = CreateWindowW(L"STATIC", L"Mumble DXGI Window", WS_OVERLAPPEDWINDOW,
 					                          CW_USEDEFAULT, CW_USEDEFAULT, 640, 480, 0,
@@ -657,9 +673,13 @@ extern "C" __declspec(dllexport) void __cdecl PrepareDXGI() {
 					desc.SwapEffect = DXGI_SWAP_EFFECT_DISCARD;
 
 					hr = pD3D10CreateDeviceAndSwapChain(pAdapter, D3D10_DRIVER_TYPE_HARDWARE, NULL, 0, D3D10_SDK_VERSION, &desc, &pSwapChain, &pDevice);
+					if (FAILED(hr))
+						ods("D3D10: pD3D10CreateDeviceAndSwapChain failure!");
 
 					if (pDevice && pSwapChain) {
 						HMODULE hRef;
+						// For VC++ the vtable is located at the base addr. of the object and each function entry is a single pointer. Since p.e. the base classes
+						// of IDXGISwapChain have a total of 8 functions the 8+Xth entry points to the Xth added function in the derived interface.
 						void ***vtbl = (void ***) pSwapChain;
 						void *pPresent = (*vtbl)[8];
 						if (! GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, (char *) pPresent, &hRef)) {
