@@ -462,6 +462,7 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 
 			if (pDst == pSelf) {
 				g.mw->updateChatBar();
+				qsDesiredChannel = c->getPath();
 			}
 
 			if (log && (pDst != pSelf) && (pDst->cChannel == pSelf->cChannel)) {
@@ -507,6 +508,7 @@ void MainWindow::msgUserRemove(const MumbleProto::UserRemove &msg) {
 	QString reason = Qt::escape(u8(msg.reason()));
 
 	if (pDst == pSelf) {
+		bRetryServer = false;
 		if (msg.ban())
 			g.l->log(Log::YouKicked, tr("You were kicked and banned from the server by %1: %2.").arg(Log::formatClientUser(pSrc, Log::Source)).arg(reason));
 		else
