@@ -90,8 +90,12 @@ OverlayClient::~OverlayClient() {
 	delete qgpiCursor;
 	delete qgpiLogo;
 
-	qlsSocket->disconnect();
-	qlsSocket->abort();
+	qlsSocket->disconnectFromServer();
+	if (!qlsSocket->waitForDisconnected(1000)) {
+		qDebug() << "OverlayClient: Failed to cleanly disconnect: " << qlsSocket->errorString();
+		qlsSocket->abort();
+	}
+	
 	qlsSocket->deleteLater();
 
 	ougUsers.reset();
@@ -447,6 +451,17 @@ void OverlayClient::readyRead() {
 
 						OverlayMsgPid *omp = & omMsg.omp;
 						uiPid = omp->pid;
+#ifdef Q_OS_WIN
+						HANDLE h = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, (DWORD)uiPid);
+						if (h) {
+							TCHAR buf[MAX_PATH];
+							if (GetModuleFileNameEx(h, 0, buf, MAX_PATH)) {
+								static_assert(sizeof(TCHAR) == sizeof(wchar_t), "Unicode build required");
+								qsProcessName = QString::fromWCharArray((const wchar_t *) buf);
+							}
+							CloseHandle(h);
+						}
+#endif
 					}
 					break;
 				case OVERLAY_MSGTYPE_FPS: {
