@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,16 +28,24 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "mumble_pch.hpp"
+
 #include "PulseAudio.h"
-#include "User.h"
-#include "Global.h"
-#include "MainWindow.h"
-#include "Timer.h"
+
 #include <sys/soundcard.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <sys/ioctl.h>
 
+#include "Global.h"
+#include "MainWindow.h"
+#include "Timer.h"
+#include "User.h"
+
+
+static const char *mumble_sink_input = "Mumble Speakers";
+static const char *mumble_echo = "Mumble Speakers (Echo)";
+
 static PulseAudioSystem *pasys = NULL;
 
 #define NBLOCKS 8
@@ -58,6 +66,7 @@ class PulseAudioOutputRegistrar : public AudioOutputRegistrar {
 		virtual AudioOutput *create();
 		virtual const QList<audioDevice> getDeviceChoices();
 		virtual void setDeviceChoice(const QVariant &, Settings &);
+		bool canMuteOthers() const;
 };
 
 class PulseAudioInit : public DeferInit {
@@ -94,12 +103,23 @@ PulseAudioSystem::PulseAudioSystem() {
 	bSourceDone=bSinkDone=bServerDone = false;
 	iDelayCache = 0;
 	bPositionalCache = false;
+	bAttenuating = false;
+	iRemainingOperations = 0;
 	bPulseIsGood = false;
 
 	pam = pa_threaded_mainloop_new();
 	pa_mainloop_api *api = pa_threaded_mainloop_get_api(pam);
 
-	pacContext = pa_context_new(api, "Mumble");
+	pa_proplist *proplist;
+
+	proplist = pa_proplist_new();
+	pa_proplist_sets(proplist, PA_PROP_APPLICATION_NAME, "Mumble");
+	pa_proplist_sets(proplist, PA_PROP_APPLICATION_ID, "net.sourceforge.mumble.mumble");
+	pa_proplist_sets(proplist, PA_PROP_APPLICATION_ICON_NAME, "mumble");
+	pa_proplist_sets(proplist, PA_PROP_MEDIA_ROLE, "phone");
+
+	pacContext = pa_context_new_with_proplist(api, NULL, proplist);
+	pa_proplist_free(proplist);
 
 	pa_context_set_subscribe_callback(pacContext, subscribe_callback, this);
 
@@ -110,9 +130,22 @@ PulseAudioSystem::PulseAudioSystem() {
 	api->defer_enable(pade, false);
 
 	pa_threaded_mainloop_start(pam);
+
+	bRunning = true;
 }
 
 PulseAudioSystem::~PulseAudioSystem() {
+	bRunning = false;
+	if (bAttenuating) {
+		qmWait.lock();
+		bAttenuating = false;
+		setVolumes();
+		bool success = qwcWait.wait(&qmWait, 1000);
+		if (! success) {
+			qWarning("PulseAudio: Shutdown timeout when attempting to restore volumes.");
+		}
+		qmWait.unlock();
+	}
 	pa_threaded_mainloop_stop(pam);
 	pa_context_disconnect(pacContext);
 	pa_context_unref(pacContext);
@@ -162,14 +195,17 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *) {
 				case PA_STREAM_TERMINATED: {
 						if (pasOutput)
 							pa_stream_unref(pasOutput);
+
 						pa_sample_spec pss = qhSpecMap.value(odev);
+						pa_channel_map pcm = qhChanMap.value(odev);
+						if ((pss.format != PA_SAMPLE_FLOAT32NE) && (pss.format != PA_SAMPLE_S16NE))
+							pss.format = PA_SAMPLE_FLOAT32NE;
 						if (pss.rate == 0)
 							pss.rate = SAMPLE_RATE;
 						if ((pss.channels == 0) || (! g.s.doPositionalAudio()))
 							pss.channels = 1;
-						if ((pss.format != PA_SAMPLE_FLOAT32NE) && (pss.format != PA_SAMPLE_S16NE))
-							pss.format = PA_SAMPLE_FLOAT32NE;
-						pasOutput = pa_stream_new(pacContext, "Mumble Speakers", &pss, NULL);
+
+						pasOutput = pa_stream_new(pacContext, mumble_sink_input, &pss, (pss.channels == 1) ? NULL : &pcm);
 						pa_stream_set_state_callback(pasOutput, stream_callback, this);
 						pa_stream_set_write_callback(pasOutput, write_callback, this);
 					}
@@ -227,11 +263,11 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *) {
 							pa_stream_unref(pasInput);
 
 						pa_sample_spec pss = qhSpecMap.value(idev);
+						if ((pss.format != PA_SAMPLE_FLOAT32NE) && (pss.format != PA_SAMPLE_S16NE))
+							pss.format = PA_SAMPLE_FLOAT32NE;
 						if (pss.rate == 0)
 							pss.rate = SAMPLE_RATE;
 						pss.channels = 1;
-						if ((pss.format != PA_SAMPLE_FLOAT32NE) && (pss.format != PA_SAMPLE_S16NE))
-							pss.format = PA_SAMPLE_FLOAT32NE;
 
 						pasInput = pa_stream_new(pacContext, "Microphone", &pss, NULL);
 						pa_stream_set_state_callback(pasInput, stream_callback, this);
@@ -286,12 +322,15 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *) {
 							pa_stream_unref(pasSpeaker);
 
 						pa_sample_spec pss = qhSpecMap.value(edev);
-						if (pss.rate == 0)
-							pss.rate = SAMPLE_RATE;
-						pss.channels = 1;
+						pa_channel_map pcm = qhChanMap.value(edev);
 						if ((pss.format != PA_SAMPLE_FLOAT32NE) && (pss.format != PA_SAMPLE_S16NE))
 							pss.format = PA_SAMPLE_FLOAT32NE;
-						pasSpeaker = pa_stream_new(pacContext, "Mumble Speakers (Echo)", &pss, NULL);
+						if (pss.rate == 0)
+							pss.rate = SAMPLE_RATE;
+						if ((pss.channels == 0) || (! g.s.bEchoMulti))
+							pss.channels = 1;
+
+						pasSpeaker = pa_stream_new(pacContext, mumble_echo, &pss, (pss.channels == 1) ? NULL : &pcm);
 						pa_stream_set_state_callback(pasSpeaker, stream_callback, this);
 						pa_stream_set_read_callback(pasSpeaker, read_callback, this);
 					}
@@ -299,7 +338,9 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *) {
 					do_start = true;
 					break;
 				case PA_STREAM_READY: {
-						if (edev != qsEchoCache) {
+						if (g.s.bEchoMulti != bEchoMultiCache) {
+							do_stop = true;
+						} else if (edev != qsEchoCache) {
 							do_stop = true;
 						}
 						break;
@@ -322,6 +363,7 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *) {
 			buff.prebuf = -1;
 			buff.fragsize = iBlockLen;
 
+			bEchoMultiCache = g.s.bEchoMulti;
 			qsEchoCache = edev;
 
 			pa_stream_connect_record(pasSpeaker, qPrintable(edev), &buff, PA_STREAM_ADJUST_LATENCY);
@@ -364,8 +406,8 @@ void PulseAudioSystem::sink_callback(pa_context *, const pa_sink_info *i, int eo
 
 	const QString name = QLatin1String(i->name);
 
-	pas->qhIndexMap.insert(name, i->index);
 	pas->qhSpecMap.insert(name, i->sample_spec);
+	pas->qhChanMap.insert(name, i->channel_map);
 	pas->qhOutput.insert(name, QLatin1String(i->description));
 	pas->qhEchoMap.insert(name, QLatin1String(i->monitor_source_name));
 }
@@ -380,8 +422,8 @@ void PulseAudioSystem::source_callback(pa_context *, const pa_source_info *i, in
 
 	const QString name = QLatin1String(i->name);
 
-	pas->qhIndexMap.insert(name, i->index);
 	pas->qhSpecMap.insert(name, i->sample_spec);
+	pas->qhChanMap.insert(name, i->channel_map);
 
 	if (i->monitor_of_sink == PA_INVALID_INDEX)
 		pas->qhInput.insert(QLatin1String(i->name), QLatin1String(i->description));
@@ -413,13 +455,23 @@ void PulseAudioSystem::read_callback(pa_stream *s, size_t bytes, void *userdata)
 	PulseAudioSystem *pas = reinterpret_cast<PulseAudioSystem *>(userdata);
 
 	size_t length = bytes;
-	const void *data;
+	const void *data = NULL;
 	pa_stream_peek(s, &data, &length);
+	if (data == NULL && length > 0) {
+		qWarning("PulseAudio: pa_stream_peek reports no data at current read index.");
+	} else if (data == NULL && length == 0) {
+		qWarning("PulseAudio: pa_stream_peek reports empty memblockq.");
+	} else if (data == NULL || length == 0) {
+		qWarning("PulseAudio: invalid pa_stream_peek state encountered.");
+		return;
+	}
 
 	AudioInputPtr ai = g.ai;
 	PulseAudioInput *pai = dynamic_cast<PulseAudioInput *>(ai.get());
 	if (! pai) {
-		pa_stream_drop(s);
+		if (length > 0) {
+			pa_stream_drop(s);
+		}
 		pas->wakeup();
 		return;
 	}
@@ -432,27 +484,33 @@ void PulseAudioSystem::read_callback(pa_stream *s, size_t bytes, void *userdata)
 			pai->iMicFreq = pss->rate;
 			pai->iMicChannels = pss->channels;
 			if (pss->format == PA_SAMPLE_FLOAT32NE)
-				pai->eMicFormat = AudioInput::SampleFloat;
+				pai->eMicFormat = PulseAudioInput::SampleFloat;
 			else
-				pai->eMicFormat = AudioInput::SampleShort;
+				pai->eMicFormat = PulseAudioInput::SampleShort;
 			pai->initializeMixer();
 		}
-		pai->addMic(data, length / pai->iMicSampleSize);
+		if (data != NULL) {
+			pai->addMic(data, length / pai->iMicSampleSize);
+		}
 	} else if (s == pas->pasSpeaker) {
 		if (!pa_sample_spec_equal(pss, &pai->pssEcho)) {
 			pai->pssEcho = *pss;
 			pai->iEchoFreq = pss->rate;
 			pai->iEchoChannels = pss->channels;
 			if (pss->format == PA_SAMPLE_FLOAT32NE)
-				pai->eEchoFormat = AudioInput::SampleFloat;
+				pai->eEchoFormat = PulseAudioInput::SampleFloat;
 			else
-				pai->eEchoFormat = AudioInput::SampleShort;
+				pai->eEchoFormat = PulseAudioInput::SampleShort;
 			pai->initializeMixer();
 		}
-		pai->addEcho(data, length / pai->iEchoSampleSize);
+		if (data != NULL) {
+			pai->addEcho(data, length / pai->iEchoSampleSize);
+		}
 	}
 
-	pa_stream_drop(s);
+	if (length > 0) {
+		pa_stream_drop(s);
+	}
 }
 
 void PulseAudioSystem::write_callback(pa_stream *s, size_t bytes, void *userdata) {
@@ -478,9 +536,9 @@ void PulseAudioSystem::write_callback(pa_stream *s, size_t bytes, void *userdata
 		pao->pss = *pss;
 		pao->pcm = *pcm;
 		if (pss->format == PA_SAMPLE_FLOAT32NE)
-			pao->eSampleFormat = AudioOutput::SampleFloat;
+			pao->eSampleFormat = PulseAudioOutput::SampleFloat;
 		else
-			pao->eSampleFormat = AudioOutput::SampleShort;
+			pao->eSampleFormat = PulseAudioOutput::SampleShort;
 		pao->iMixerFreq = pss->rate;
 		pao->iChannels = pss->channels;
 		unsigned int chanmasks[pss->channels];
@@ -531,17 +589,186 @@ void PulseAudioSystem::write_callback(pa_stream *s, size_t bytes, void *userdata
 
 	const unsigned int iSampleSize = pao->iSampleSize;
 	const unsigned int samples = bytes / iSampleSize;
+	bool oldAttenuation = pas->bAttenuating;
 
-	if (! pao->mix(buffer, samples))
+	// do we have some mixed output?
+	if (pao->mix(buffer, samples)) {
+		// attenuate if instructed to or it's in settings
+		pas->bAttenuating = (g.bAttenuateOthers || g.s.bAttenuateOthers);
+
+	} else {
 		memset(buffer, 0, bytes);
+
+		// attenuate if intructed to (self-activated)
+		pas->bAttenuating = g.bAttenuateOthers;
+	}
+
+	// if the attenuation state has changed
+	if (oldAttenuation != pas->bAttenuating) {
+		pas->setVolumes();
+	}
+
 	pa_stream_write(s, buffer, iSampleSize * samples, NULL, 0, PA_SEEK_RELATIVE);
 }
 
+void PulseAudioSystem::volume_sink_input_list_callback(pa_context *c, const pa_sink_input_info *i, int eol, void *userdata) {
+	PulseAudioSystem *pas = reinterpret_cast<PulseAudioSystem *>(userdata);
+
+	if (eol == 0) {
+		// ensure we're not attenuating ourselves!
+		if (strcmp(i->name, mumble_sink_input) != 0) {
+			// create a new entry
+			PulseAttenuation patt;
+			patt.index = i->index;
+			patt.name = QLatin1String(i->name);
+			patt.stream_restore_id = QLatin1String(pa_proplist_gets(i->proplist, "module-stream-restore.id"));
+			patt.normal_volume = i->volume;
+
+			// calculate the attenuated volume
+			pa_volume_t adj = static_cast<pa_volume_t>(PA_VOLUME_NORM * g.s.fOtherVolume);
+			pa_sw_cvolume_multiply_scalar(&patt.attenuated_volume, &i->volume, adj);
+
+			// set it on the sink input
+			pa_operation_unref(pa_context_set_sink_input_volume(c, i->index, &patt.attenuated_volume, NULL, NULL));
+
+			// store it
+			pas->qhVolumes[i->index] = patt;
+		}
+
+	} else if (eol < 0) {
+		qWarning("PulseAudio: Sink input introspection error.");
+	}
+}
+
+void PulseAudioSystem::restore_sink_input_list_callback(pa_context *c, const pa_sink_input_info *i, int eol, void *userdata) {
+	PulseAudioSystem *pas = reinterpret_cast<PulseAudioSystem *>(userdata);
+
+	if (eol == 0) {
+		// if we were tracking this specific sink previously
+		if (pas->qhVolumes.contains(i->index)) {
+			// and if it has the attenuated volume we applied to it
+			if (pa_cvolume_equal(&i->volume, &pas->qhVolumes[i->index].attenuated_volume) != 0) {
+				// mark it as matched
+				pas->qlMatchedSinks.append(i->index);
+
+				// reset the volume to normal
+				pas->iRemainingOperations++;
+				pa_operation_unref(pa_context_set_sink_input_volume(c, i->index, &pas->qhVolumes[i->index].normal_volume, restore_volume_success_callback, pas));
+			}
+
+		// otherwise, save for matching at the end of iteration
+		} else {
+			QString restore_id = QLatin1String(pa_proplist_gets(i->proplist, "module-stream-restore.id"));
+			PulseAttenuation patt;
+			patt.index = i->index;
+			patt.normal_volume = i->volume;
+			pas->qhUnmatchedSinks[restore_id] = patt;
+		}
+
+	} else if (eol < 0) {
+		qWarning("PulseAudio: Sink input introspection error.");
+
+	} else {
+		// build a list of missing streams by iterating our active list
+		QHash<uint32_t, PulseAttenuation>::const_iterator it;
+		for (it = pas->qhVolumes.constBegin(); it != pas->qhVolumes.constEnd(); ++it) {
+			// skip if previously matched
+			if (pas->qlMatchedSinks.contains(it.key())) {
+				continue;
+			}
+
+			// check if the restore id matches. the only case where this would
+			// happen is if the application was reopened during attenuation.
+			if (pas->qhUnmatchedSinks.contains(it.value().stream_restore_id)) {
+				PulseAttenuation active_sink = pas->qhUnmatchedSinks[it.value().stream_restore_id];
+				// if the volume wasn't changed from our attenuation
+				if (pa_cvolume_equal(&active_sink.normal_volume, &it.value().attenuated_volume) != 0) {
+					// reset the volume to normal
+					pas->iRemainingOperations++;
+					pa_operation_unref(pa_context_set_sink_input_volume(c, active_sink.index, &it.value().normal_volume, restore_volume_success_callback, pas));
+				}
+				continue;
+			}
+
+			// at this point, we don't know what happened to the sink. add
+			// it to a list to check the stream restore database for.
+			pas->qhMissingSinks[it.value().stream_restore_id] = it.value();
+		}
+
+		// clean up
+		pas->qlMatchedSinks.clear();
+		pas->qhUnmatchedSinks.clear();
+		pas->qhVolumes.clear();
+
+		// if we had missing sinks, check the stream restore database
+		// to see if we can find and update them.
+		if (pas->qhMissingSinks.count() > 0) {
+			pas->iRemainingOperations++;
+			pa_operation_unref(pa_ext_stream_restore_read(c, stream_restore_read_callback, pas));
+		}
+
+		// trigger the volume completion callback;
+		// necessary so that shutdown actions are called
+		restore_volume_success_callback(c, 1, pas);
+	}
+}
+
+void PulseAudioSystem::stream_restore_read_callback(pa_context *c, const pa_ext_stream_restore_info *i, int eol, void *userdata) {
+	PulseAudioSystem *pas = reinterpret_cast<PulseAudioSystem *>(userdata);
+
+	if (eol == 0) {
+		QString name = QLatin1String(i->name);
+
+		// were we looking for this restoration?
+		if (pas->qhMissingSinks.contains(name)) {
+			// make sure it still has the volume we gave it
+			if (pa_cvolume_equal(&pas->qhMissingSinks[name].attenuated_volume, &i->volume) != 0) {
+				// update the stream restore record
+				pa_ext_stream_restore_info restore = *i;
+				restore.volume = pas->qhMissingSinks[name].normal_volume;
+				pas->iRemainingOperations++;
+				pa_operation_unref(pa_ext_stream_restore_write(c, PA_UPDATE_REPLACE, &restore, 1, 1, restore_volume_success_callback, pas));
+			}
+
+			pas->qhMissingSinks.remove(name);
+		}
+
+	} else if (eol < 0) {
+		qWarning("PulseAudio: Couldn't read stream restore database.");
+		pas->qhMissingSinks.clear();
+
+	} else {
+		// verify missing list is empty
+		if (pas->qhMissingSinks.count() > 0) {
+			qWarning("PulseAudio: Failed to match %d stream(s).", pas->qhMissingSinks.count());
+			pas->qhMissingSinks.clear();
+		}
+
+		// trigger the volume completion callback;
+		// necessary so that shutdown actions are called
+		restore_volume_success_callback(c, 1, pas);
+	}
+}
+
+void PulseAudioSystem::restore_volume_success_callback(pa_context *c, int success, void *userdata) {
+	PulseAudioSystem *pas = reinterpret_cast<PulseAudioSystem *>(userdata);
+
+	pas->iRemainingOperations--;
+
+	// if there are no more pending volume adjustments and we're shutting down,
+	// let the main thread know
+	if (! pas->bRunning && pas->iRemainingOperations == 0) {
+		pas->qwcWait.wakeAll();
+	}
+}
+
 void PulseAudioSystem::query() {
 	bSourceDone=bSinkDone=bServerDone = false;
 	qhInput.clear();
 	qhOutput.clear();
 	qhEchoMap.clear();
+	qhSpecMap.clear();
+	qhChanMap.clear();
 	qhInput.insert(QString(), tr("Default Input"));
 	qhOutput.insert(QString(), tr("Default Output"));
 	pa_operation_unref(pa_context_get_server_info(pacContext, server_callback, this));
@@ -550,6 +777,21 @@ void PulseAudioSystem::query() {
 	wakeup();
 }
 
+void PulseAudioSystem::setVolumes() {
+	// set attenuation state and volumes
+	if (bAttenuating) {
+		// ensure the volume map is empty, otherwise it may be dangerous to change
+		if (qhVolumes.empty()) {
+			// set the new per-application volumes and store the old ones
+			pa_operation_unref(pa_context_get_sink_input_info_list(pacContext, volume_sink_input_list_callback, this));
+		}
+	// clear attenuation state and restore normal volumes
+	} else {
+		iRemainingOperations++;
+		pa_operation_unref(pa_context_get_sink_input_info_list(pacContext, restore_sink_input_list_callback, this));
+	}
+}
+
 void PulseAudioSystem::contextCallback(pa_context *c) {
 	Q_ASSERT(c == pacContext);
 	switch (pa_context_get_state(c)) {
@@ -635,6 +877,10 @@ void PulseAudioOutputRegistrar::setDeviceChoice(const QVariant &choice, Settings
 	s.qsPulseAudioOutput = choice.toString();
 }
 
+bool PulseAudioOutputRegistrar::canMuteOthers() const {
+	return true;
+}
+
 PulseAudioInput::PulseAudioInput() {
 	memset(&pssMic, 0, sizeof(pssMic));
 	memset(&pssEcho, 0, sizeof(pssEcho));
