@@ -37,6 +37,8 @@
 #include "Cert.h"
 #include "Log.h"
 #include "Global.h"
+#include "SSL.h"
+
 #include "../../overlay/overlay.h"
 #include "../../overlay/overlay_blacklist.h"
 
@@ -323,6 +325,8 @@ Settings::Settings() {
 
 	iOutputDelay = 5;
 
+	bASIOEnable = true;
+
 	qsALSAInput=QLatin1String("default");
 	qsALSAOutput=QLatin1String("default");
 
@@ -360,6 +364,7 @@ Settings::Settings() {
 	iMaxImageWidth = 1024; // Allow 1024x1024 resolution
 	iMaxImageHeight = 1024;
 	bSuppressIdentity = false;
+	qsSslCiphers = MumbleSSL::defaultOpenSSLCipherString();
 
 	// Accessibility
 	bHighContrast = false;
@@ -388,6 +393,8 @@ Settings::Settings() {
 
 	bShortcutEnable = true;
 	bSuppressMacEventTapWarning = false;
+	bEnableEvdev = false;
+	bEnableXInput2 = true;
 
 	for (int i=Log::firstMsgType; i<=Log::lastMsgType; ++i)
 		qmMessages.insert(i, Settings::LogConsole | Settings::LogBalloon | Settings::LogTTS);
@@ -578,6 +585,7 @@ void Settings::load(QSettings* settings_ptr) {
 	SAVELOAD(iJitterBufferSize, "net/jitterbuffer");
 	SAVELOAD(iFramesPerPacket, "net/framesperpacket");
 
+	SAVELOAD(bASIOEnable, "asio/enable");
 	SAVELOAD(qsASIOclass, "asio/class");
 	SAVELOAD(qlASIOmic, "asio/mic");
 	SAVELOAD(qlASIOspeaker, "asio/speaker");
@@ -624,6 +632,9 @@ void Settings::load(QSettings* settings_ptr) {
 	SAVELOAD(iMaxImageHeight, "net/maximageheight");
 	SAVELOAD(qsRegionalHost, "net/region");
 
+	// Network settings - SSL
+	SAVELOAD(qsSslCiphers, "net/sslciphers");
+
 	SAVELOAD(bExpert, "ui/expert");
 	SAVELOAD(qsLanguage, "ui/language");
 	SAVELOAD(qsStyle, "ui/style");
@@ -685,6 +696,8 @@ void Settings::load(QSettings* settings_ptr) {
 
 	SAVELOAD(bShortcutEnable, "shortcut/enable");
 	SAVELOAD(bSuppressMacEventTapWarning, "shortcut/mac/suppresswarning");
+	SAVELOAD(bEnableEvdev, "shortcut/linux/evdev/enable");
+	SAVELOAD(bEnableXInput2, "shortcut/x11/xinput2/enable");
 
 	int nshorts = settings_ptr->beginReadArray(QLatin1String("shortcuts"));
 	for (int i=0; i<nshorts; i++) {
@@ -864,6 +877,7 @@ void Settings::save() {
 	SAVELOAD(iJitterBufferSize, "net/jitterbuffer");
 	SAVELOAD(iFramesPerPacket, "net/framesperpacket");
 
+	SAVELOAD(bASIOEnable, "asio/enable");
 	SAVELOAD(qsASIOclass, "asio/class");
 	SAVELOAD(qlASIOmic, "asio/mic");
 	SAVELOAD(qlASIOspeaker, "asio/speaker");
@@ -909,6 +923,9 @@ void Settings::save() {
 	SAVELOAD(iMaxImageHeight, "net/maximageheight");
 	SAVELOAD(qsRegionalHost, "net/region");
 
+	// Network settings - SSL
+	SAVELOAD(qsSslCiphers, "net/sslciphers");
+
 	SAVELOAD(bExpert, "ui/expert");
 	SAVELOAD(qsLanguage, "ui/language");
 	SAVELOAD(qsStyle, "ui/style");
@@ -967,6 +984,8 @@ void Settings::save() {
 
 	SAVELOAD(bShortcutEnable, "shortcut/enable");
 	SAVELOAD(bSuppressMacEventTapWarning, "shortcut/mac/suppresswarning");
+	SAVELOAD(bEnableEvdev, "shortcut/linux/evdev/enable");
+	SAVELOAD(bEnableXInput2, "shortcut/x11/xinput2/enable");
 
 	settings_ptr->beginWriteArray(QLatin1String("shortcuts"));
 	int idx = 0;
