@@ -207,9 +207,12 @@ struct Settings {
 	QString qsPulseAudioInput, qsPulseAudioOutput;
 	QString qsOSSInput, qsOSSOutput;
 	int iPortAudioInput, iPortAudioOutput;
+
+	bool bASIOEnable;
 	QString qsASIOclass;
 	QList<QVariant> qlASIOmic;
 	QList<QVariant> qlASIOspeaker;
+
 	QString qsCoreAudioInput, qsCoreAudioOutput;
 	QString qsWASAPIInput, qsWASAPIOutput;
 	QByteArray qbaDXInput, qbaDXOutput;
@@ -230,6 +233,8 @@ struct Settings {
 
 	bool bShortcutEnable;
 	bool bSuppressMacEventTapWarning;
+	bool bEnableEvdev;
+	bool bEnableXInput2;
 	QList<Shortcut> qlShortcuts;
 
 	enum MessageLog { LogNone = 0x00, LogConsole = 0x01, LogTTS = 0x02, LogBalloon = 0x04, LogSoundfile = 0x08};
@@ -283,6 +288,9 @@ struct Settings {
 	unsigned short usProxyPort;
 	QString qsRegionalHost;
 
+	// Network settings - SSL
+	QString qsSslCiphers;
+
 	static const int ciDefaultMaxImageSize = 50 * 1024; // Restrict to 50KiB as a default
 	int iMaxImageSize;
 	int iMaxImageWidth;
