@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,10 +28,10 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef _TEXTTOSPEECH_H
-#define _TEXTTOSPEECH_H
+#ifndef TEXTTOSPEECH_H_
+#define TEXTTOSPEECH_H_
 
-#include "mumble_pch.hpp"
+#include <QtCore/QObject>
 
 class TextToSpeechPrivate;
 
@@ -55,6 +55,4 @@ class TextToSpeech : public QObject {
 		TextToSpeechPrivate *d;
 };
 
-#else
-class TextToSpeech;
 #endif
