@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,14 +28,17 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include "UserModel.h"
+#include "mumble_pch.hpp"
+
 #include "UserView.h"
-#include "MainWindow.h"
-#include "ClientUser.h"
+
 #include "Channel.h"
-#include "ServerHandler.h"
+#include "ClientUser.h"
 #include "Log.h"
 #include "Global.h"
+#include "MainWindow.h"
+#include "ServerHandler.h"
+#include "UserModel.h"
 
 /*!
   \fn bool UserView::event(QEvent *evt)
@@ -175,6 +178,8 @@ void UserView::mouseReleaseEvent(QMouseEvent *evt) {
 
 			if (cu) {
 				// Calculate pixel offset of comment flag
+				if (cu->bLocalIgnore)
+					offset += 18;
 				if (cu->bRecording)
 					offset += 18;
 				if (cu->bPrioritySpeaker)
