@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,11 +28,13 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "mumble_pch.hpp"
+
 #include "WASAPI.h"
-#include "User.h"
+#include "WASAPINotificationClient.h"
 #include "Global.h"
-#include "MainWindow.h"
 #include "Timer.h"
+#include "User.h"
 
 
 // Now that Win7 is published, which includes public versions of these
@@ -56,6 +58,7 @@ class WASAPIInputRegistrar : public AudioInputRegistrar {
 		virtual const QList<audioDevice> getDeviceChoices();
 		virtual void setDeviceChoice(const QVariant &, Settings &);
 		virtual bool canEcho(const QString &) const;
+		virtual bool canExclusive() const;
 };
 
 class WASAPIOutputRegistrar : public AudioOutputRegistrar {
@@ -65,29 +68,27 @@ class WASAPIOutputRegistrar : public AudioOutputRegistrar {
 		virtual const QList<audioDevice> getDeviceChoices();
 		virtual void setDeviceChoice(const QVariant &, Settings &);
 		bool canMuteOthers() const;
+		virtual bool canExclusive() const;
 };
 
 class WASAPIInit : public DeferInit {
 		WASAPIInputRegistrar *wirReg;
 		WASAPIOutputRegistrar *worReg;
 	public:
+		WASAPIInit() : wirReg(NULL), worReg(NULL) { }
 		void initialize();
 		void destroy();
 };
 
 static WASAPIInit wasapiinit;
 
+extern bool bIsWin7, bIsVistaSP1;
+
 void WASAPIInit::initialize() {
 	wirReg = NULL;
 	worReg = NULL;
 
-	OSVERSIONINFOEXW ovi;
-	memset(&ovi, 0, sizeof(ovi));
-
-	ovi.dwOSVersionInfoSize=sizeof(ovi);
-	GetVersionEx(reinterpret_cast<OSVERSIONINFOW *>(&ovi));
-
-	if ((ovi.dwMajorVersion < 6) || (ovi.dwBuildNumber < 6001)) {
+	if (! bIsVistaSP1) {
 		qWarning("WASAPIInit: Requires Vista SP1");
 		return;
 	}
@@ -104,10 +105,8 @@ void WASAPIInit::initialize() {
 }
 
 void WASAPIInit::destroy() {
-	if (wirReg)
-		delete wirReg;
-	if (worReg)
-		delete worReg;
+	delete wirReg;
+	delete worReg;
 }
 
 
@@ -130,6 +129,10 @@ bool WASAPIInputRegistrar::canEcho(const QString &outputsys) const {
 	return (outputsys == name);
 }
 
+bool WASAPIInputRegistrar::canExclusive() const {
+	return true;
+}
+
 WASAPIOutputRegistrar::WASAPIOutputRegistrar() : AudioOutputRegistrar(QLatin1String("WASAPI"), 10) {
 }
 
@@ -149,6 +152,10 @@ bool WASAPIOutputRegistrar::canMuteOthers() const {
 	return true;
 }
 
+bool WASAPIOutputRegistrar::canExclusive() const {
+	return true;
+}
+
 const QHash<QString, QString> WASAPISystem::getInputDevices() {
 	return getDevices(eCapture);
 }
@@ -168,11 +175,11 @@ const QHash<QString, QString> WASAPISystem::getDevices(EDataFlow dataflow) {
 	hr = CoCreateInstance(__uuidof(MMDeviceEnumerator), NULL, CLSCTX_ALL, __uuidof(IMMDeviceEnumerator), reinterpret_cast<void **>(&pEnumerator));
 
 	if (! pEnumerator || FAILED(hr)) {
-		qWarning("WASAPI: Failed to instatiate enumerator");
+		qWarning("WASAPI: Failed to instatiate enumerator: hr=0x%08lx", hr);
 	} else {
 		hr = pEnumerator->EnumAudioEndpoints(dataflow, DEVICE_STATE_ACTIVE, &pCollection);
 		if (! pCollection || FAILED(hr)) {
-			qWarning("WASAPI: Failed to enumerate");
+			qWarning("WASAPI: Failed to enumerate: hr=0x%08lx", hr);
 		} else {
 			devices.insert(QString(), tr("Default Device"));
 
@@ -209,7 +216,7 @@ const QHash<QString, QString> WASAPISystem::getDevices(EDataFlow dataflow) {
 	return devices;
 }
 
-const QList<audioDevice> WASAPISystem::mapToDevice(const QHash<QString, QString> devs, const QString &match) {
+const QList<audioDevice> WASAPISystem::mapToDevice(const QHash<QString, QString>& devs, const QString& match) {
 	QList<audioDevice> qlReturn;
 
 	QStringList qlDevices = devs.keys();
@@ -227,7 +234,6 @@ const QList<audioDevice> WASAPISystem::mapToDevice(const QHash<QString, QString>
 }
 
 WASAPIInput::WASAPIInput() {
-	bRunning = true;
 };
 
 WASAPIInput::~WASAPIInput() {
@@ -235,9 +241,65 @@ WASAPIInput::~WASAPIInput() {
 	wait();
 }
 
-void WASAPIInput::run() {
+static IMMDevice *openNamedOrDefaultDevice(const QString& name, EDataFlow dataFlow, ERole role) {
 	HRESULT hr;
 	IMMDeviceEnumerator *pEnumerator = NULL;
+	IMMDevice *pDevice = NULL;
+
+	hr = CoCreateInstance(__uuidof(MMDeviceEnumerator), NULL, CLSCTX_ALL, __uuidof(IMMDeviceEnumerator), reinterpret_cast<void **>(&pEnumerator));
+	if (!pEnumerator || FAILED(hr)) {
+		qWarning("WASAPI: Failed to instantiate enumerator: hr=0x%08lx", hr);
+		return NULL;
+	}
+
+	// Try to find a device pointer for |name|.
+	if (!name.isEmpty()) {
+		STACKVAR(wchar_t, devname, name.length() + 1);
+		int len = name.toWCharArray(devname);
+		devname[len] = 0;
+		hr = pEnumerator->GetDevice(devname, &pDevice);
+		if (FAILED(hr)) {
+			qWarning("WASAPI: Failed to open selected device %s %ls (df=%d, e=%d, hr=0x%08lx), falling back to default", qPrintable(name), devname, dataFlow, role, hr);
+		} else {
+			WASAPINotificationClient::get().enlistDeviceAsUsed(devname);
+		}
+	}
+
+	// Use the default device if |pDevice| is still NULL.
+	// We retrieve the actual device name for the currently selected default device and
+	// open the device by it's real name to work around triggering the automatic
+	// ducking behavior.
+	if (!pDevice) {
+		hr = pEnumerator->GetDefaultAudioEndpoint(dataFlow, role, &pDevice);
+		if (FAILED(hr)) {
+			qWarning("WASAPI: Failed to open device: df=%d, e=%d, hr=0x%08lx", dataFlow, role, hr);
+			goto cleanup;
+		}
+		wchar_t *devname = NULL;
+		hr = pDevice->GetId(&devname);
+		if (FAILED(hr)) {
+			qWarning("WASAPI: Failed to query device: df=%d, e=%d, hr=0x%08lx", dataFlow, role, hr);
+			goto cleanup;
+		}
+		pDevice->Release();
+		hr = pEnumerator->GetDevice(devname, &pDevice);
+		if (FAILED(hr)) {
+			qWarning("WASAPI: Failed to reopen default device: df=%d, e=%d, hr=0x%08lx", dataFlow, role, hr);
+			goto cleanup;
+		}
+		WASAPINotificationClient::get().enlistDefaultDeviceAsUsed(devname);
+		CoTaskMemFree(devname);
+	}
+
+cleanup:
+	if (pEnumerator)
+		pEnumerator->Release();
+
+	return pDevice;
+}
+
+void WASAPIInput::run() {
+	HRESULT hr;
 	IMMDevice *pMicDevice = NULL;
 	IAudioClient *pMicAudioClient = NULL;
 	IAudioCaptureClient *pMicCaptureClient = NULL;
@@ -246,6 +308,7 @@ void WASAPIInput::run() {
 	IAudioCaptureClient *pEchoCaptureClient = NULL;
 	WAVEFORMATEX *micpwfx = NULL, *echopwfx = NULL;
 	WAVEFORMATEXTENSIBLE *micpwfxe = NULL, *echopwfxe = NULL;
+	WAVEFORMATEXTENSIBLE wfe;
 	UINT32 bufferFrameCount;
 	UINT32 numFramesAvailable;
 	UINT32 numFramesLeft;
@@ -259,7 +322,10 @@ void WASAPIInput::run() {
 	DWORD dwTaskIndex = 0;
 	HANDLE hMmThread;
 	float *tbuff = NULL;
+	short *sbuff = NULL;
 	bool doecho = g.s.doEcho();
+	REFERENCE_TIME def, min, latency, want;
+	bool exclusive = false;
 
 	CoInitialize(NULL);
 
@@ -270,115 +336,96 @@ void WASAPIInput::run() {
 		qWarning("WASAPIInput: Failed to set Pro Audio thread priority");
 	}
 
-	hr = CoCreateInstance(__uuidof(MMDeviceEnumerator), NULL, CLSCTX_ALL, __uuidof(IMMDeviceEnumerator), reinterpret_cast<void **>(&pEnumerator));
-
-	if (! pEnumerator || FAILED(hr)) {
-		qWarning("WASAPIInput: Failed to instatiate enumerator");
-		return;
-	}
-
-	if (! g.s.qsWASAPIInput.isEmpty()) {
-		STACKVAR(wchar_t, devname, g.s.qsWASAPIInput.length() + 1);
-		int len = g.s.qsWASAPIInput.toWCharArray(devname);
-		devname[len] = 0;
-		hr = pEnumerator->GetDevice(devname, &pMicDevice);
-		if (FAILED(hr)) {
-			qWarning("WASAPIInput: Failed to open selected input device %s %ls, falling back to default", qPrintable(g.s.qsWASAPIInput), devname);
-		}
-	}
-
-	if (! pMicDevice) {
-		hr = pEnumerator->GetDefaultAudioEndpoint(eCapture, eCommunications, &pMicDevice);
-		if (FAILED(hr)) {
-			qWarning("WASAPIInput: Failed to open input device");
-			goto cleanup;
-		}
-		wchar_t *devname = NULL;
-		hr = pMicDevice->GetId(&devname);
-		if (FAILED(hr)) {
-			qWarning("WASAPIInput: Failed to query input device");
-			goto cleanup;
-		}
-		pMicDevice->Release();
-		hr = pEnumerator->GetDevice(devname, &pMicDevice);
-		if (FAILED(hr)) {
-			qWarning("WASAPIInput: Failed to reopen default input device");
-			goto cleanup;
-		}
-		CoTaskMemFree(devname);
-	}
+	// Open mic device.
+	pMicDevice = openNamedOrDefaultDevice(g.s.qsWASAPIInput, eCapture, eCommunications);
+	if (!pMicDevice)
+		goto cleanup;
 
+	// Open echo capture device.
 	if (doecho) {
-		if (! g.s.qsWASAPIOutput.isEmpty()) {
-			STACKVAR(wchar_t, devname, g.s.qsWASAPIOutput.length());
-			g.s.qsWASAPIOutput.toWCharArray(devname);
-			hr = pEnumerator->GetDevice(devname, &pEchoDevice);
-			if (FAILED(hr)) {
-				qWarning("WASAPIInput: Failed to open selected echo device, falling back to default");
-			}
-		}
-
-		if (! pEchoDevice) {
-			hr = pEnumerator->GetDefaultAudioEndpoint(eRender, eCommunications, &pEchoDevice);
-			if (FAILED(hr)) {
-				qWarning("WASAPIInput: Failed to open echo device");
-				goto cleanup;
-			}
-			wchar_t *devname = NULL;
-			hr = pEchoDevice->GetId(&devname);
-			if (FAILED(hr)) {
-				qWarning("WASAPIInput: Failed to query echo device");
-				goto cleanup;
-			}
-			pEchoDevice->Release();
-			hr = pEnumerator->GetDevice(devname, &pEchoDevice);
-			if (FAILED(hr)) {
-				qWarning("WASAPIInput: Failed to reopen default echo device");
-				goto cleanup;
-			}
-			CoTaskMemFree(devname);
-		}
+		pEchoDevice = openNamedOrDefaultDevice(g.s.qsWASAPIOutput, eRender, eCommunications);
+		if (!pEchoDevice)
+			doecho = false;
 	}
 
-	pEnumerator->Release();
-	pEnumerator = NULL;
-
 	hr = pMicDevice->Activate(__uuidof(IAudioClient), CLSCTX_ALL, NULL, (void **) &pMicAudioClient);
 	if (FAILED(hr)) {
-		qWarning("WASAPIInput: Activate Mic AudioClient failed");
+		qWarning("WASAPIInput: Activate Mic AudioClient failed: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 
-	hr = pMicAudioClient->GetMixFormat(&micpwfx);
-	micpwfxe = reinterpret_cast<WAVEFORMATEXTENSIBLE *>(micpwfx);
+	def = min = latency = 0;
 
-	if (FAILED(hr) || (micpwfx->wBitsPerSample != (sizeof(float) * 8)) || (micpwfxe->SubFormat != KSDATAFORMAT_SUBTYPE_IEEE_FLOAT)) {
-		qWarning("WASAPIInput: Mic Subformat is not IEEE Float");
-		goto cleanup;
+	pMicAudioClient->GetDevicePeriod(&def, &min);
+
+	want = qMax<REFERENCE_TIME>(min, 100000);
+	qWarning("WASAPIInput: Latencies %lld %lld => %lld", def, min, want);
+
+	if (g.s.bExclusiveInput && ! doecho) {
+		for (int channels = 1; channels<=2; ++channels) {
+			ZeroMemory(&wfe, sizeof(wfe));
+			wfe.Format.cbSize = 0;
+			wfe.Format.wFormatTag = WAVE_FORMAT_PCM;
+			wfe.Format.nChannels = channels;
+			wfe.Format.nSamplesPerSec = 48000;
+			wfe.Format.wBitsPerSample = 16;
+			wfe.Format.nBlockAlign = wfe.Format.nChannels * wfe.Format.wBitsPerSample / 8;
+			wfe.Format.nAvgBytesPerSec = wfe.Format.nBlockAlign * wfe.Format.nSamplesPerSec;
+
+			micpwfxe = &wfe;
+			micpwfx = reinterpret_cast<WAVEFORMATEX *>(&wfe);
+
+			hr = pMicAudioClient->Initialize(AUDCLNT_SHAREMODE_EXCLUSIVE, AUDCLNT_STREAMFLAGS_EVENTCALLBACK, want, want, micpwfx, NULL);
+			if (SUCCEEDED(hr)) {
+				eMicFormat = SampleShort;
+				exclusive = true;
+				qWarning("WASAPIInput: Successfully opened exclusive mode");
+				break;
+			}
+
+			micpwfxe = NULL;
+			micpwfx = NULL;
+		}
 	}
 
-	hr = pMicAudioClient->Initialize(AUDCLNT_SHAREMODE_SHARED, AUDCLNT_STREAMFLAGS_EVENTCALLBACK, 0, 0, micpwfx, NULL);
-	if (FAILED(hr)) {
-		qWarning("WASAPIInput: Mic Initialize failed");
-		goto cleanup;
+	if (!  micpwfxe) {
+		if (g.s.bExclusiveInput)
+			qWarning("WASAPIInput: Failed to open exclusive mode.");
+
+		hr = pMicAudioClient->GetMixFormat(&micpwfx);
+		micpwfxe = reinterpret_cast<WAVEFORMATEXTENSIBLE *>(micpwfx);
+
+		if (FAILED(hr) || (micpwfx->wBitsPerSample != (sizeof(float) * 8)) || (micpwfxe->SubFormat != KSDATAFORMAT_SUBTYPE_IEEE_FLOAT)) {
+			qWarning("WASAPIInput: Mic Subformat is not IEEE Float: hr=0x%08lx", hr);
+			goto cleanup;
+		}
+
+		hr = pMicAudioClient->Initialize(AUDCLNT_SHAREMODE_SHARED, AUDCLNT_STREAMFLAGS_EVENTCALLBACK, 0, 0, micpwfx, NULL);
+		if (FAILED(hr)) {
+			qWarning("WASAPIInput: Mic Initialize failed: hr=0x%08lx", hr);
+			goto cleanup;
+		}
 	}
 
+	pMicAudioClient->GetStreamLatency(&latency);
 	hr = pMicAudioClient->GetBufferSize(&bufferFrameCount);
+	qWarning("WASAPIInput: Stream Latency %lld (%d)", latency, bufferFrameCount);
+
 	hr = pMicAudioClient->GetService(__uuidof(IAudioCaptureClient), (void**)&pMicCaptureClient);
 	if (FAILED(hr)) {
-		qWarning("WASAPIInput: Mic GetService failed");
+		qWarning("WASAPIInput: Mic GetService failed: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 
 	pMicAudioClient->SetEventHandle(hEvent);
 	if (FAILED(hr)) {
-		qWarning("WASAPIInput: Failed to set mic event");
+		qWarning("WASAPIInput: Failed to set mic event: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 
 	hr = pMicAudioClient->Start();
 	if (FAILED(hr)) {
-		qWarning("WASAPIInput: Failed to start mic");
+		qWarning("WASAPIInput: Failed to start mic: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 
@@ -388,7 +435,7 @@ void WASAPIInput::run() {
 	if (doecho) {
 		hr = pEchoDevice->Activate(__uuidof(IAudioClient), CLSCTX_ALL, NULL, (void **) &pEchoAudioClient);
 		if (FAILED(hr)) {
-			qWarning("WASAPIInput: Activate Echo AudioClient failed");
+			qWarning("WASAPIInput: Activate Echo AudioClient failed: hr=0x%08lx", hr);
 			goto cleanup;
 		}
 
@@ -396,32 +443,32 @@ void WASAPIInput::run() {
 		echopwfxe = reinterpret_cast<WAVEFORMATEXTENSIBLE *>(echopwfx);
 
 		if (FAILED(hr) || (echopwfx->wBitsPerSample != (sizeof(float) * 8)) || (echopwfxe->SubFormat != KSDATAFORMAT_SUBTYPE_IEEE_FLOAT)) {
-			qWarning("WASAPIInput: Echo Subformat is not IEEE Float");
+			qWarning("WASAPIInput: Echo Subformat is not IEEE Float: hr=0x%08lx", hr);
 			goto cleanup;
 		}
 
 		hr = pEchoAudioClient->Initialize(AUDCLNT_SHAREMODE_SHARED, AUDCLNT_STREAMFLAGS_EVENTCALLBACK | AUDCLNT_STREAMFLAGS_LOOPBACK, 0, 0, echopwfx, NULL);
 		if (FAILED(hr)) {
-			qWarning("WASAPIInput: Echo Initialize failed");
+			qWarning("WASAPIInput: Echo Initialize failed: hr=0x%08lx", hr);
 			goto cleanup;
 		}
 
 		hr = pEchoAudioClient->GetBufferSize(&bufferFrameCount);
 		hr = pEchoAudioClient->GetService(__uuidof(IAudioCaptureClient), (void**)&pEchoCaptureClient);
 		if (FAILED(hr)) {
-			qWarning("WASAPIInput: Echo GetService failed");
+			qWarning("WASAPIInput: Echo GetService failed: hr=0x%08lx", hr);
 			goto cleanup;
 		}
 
 		pEchoAudioClient->SetEventHandle(hEvent);
 		if (FAILED(hr)) {
-			qWarning("WASAPIInput: Failed to set echo event");
+			qWarning("WASAPIInput: Failed to set echo event: hr=0x%08lx", hr);
 			goto cleanup;
 		}
 
 		hr = pEchoAudioClient->Start();
 		if (FAILED(hr)) {
-			qWarning("WASAPIInput: Failed to start Echo");
+			qWarning("WASAPIInput: Failed to start Echo: hr=0x%08lx", hr);
 			goto cleanup;
 		}
 
@@ -432,61 +479,99 @@ void WASAPIInput::run() {
 	initializeMixer();
 
 	allocLength = (iMicLength / 2) * micpwfx->nChannels;
-	tbuff = new float[allocLength];
 
-	while (bRunning && ! FAILED(hr)) {
-		hr = pMicCaptureClient->GetNextPacketSize(&micPacketLength);
-		if (! FAILED(hr) && iEchoChannels)
-			hr = pEchoCaptureClient->GetNextPacketSize(&echoPacketLength);
-		if (FAILED(hr))
-			goto cleanup;
+	if (exclusive) {
+		sbuff = new short[allocLength];
+		while (bRunning && ! FAILED(hr)) {
+			hr = pMicCaptureClient->GetBuffer(&pData, &numFramesAvailable, &flags, &devicePosition, &qpcPosition);
+			if (hr != AUDCLNT_S_BUFFER_EMPTY) {
+				if (FAILED(hr))
+					goto cleanup;
 
-		while ((micPacketLength > 0) || (echoPacketLength > 0)) {
-			if (echoPacketLength > 0) {
-				hr = pEchoCaptureClient->GetBuffer(&pData, &numFramesAvailable, &flags, &devicePosition, &qpcPosition);
 				numFramesLeft = numFramesAvailable;
-				if (FAILED(hr))
-					goto cleanup;
-
-				UINT32 nFrames = numFramesAvailable * echopwfx->nChannels;
-				if (nFrames > allocLength) {
-					delete [] tbuff;
-					allocLength = nFrames;
-					tbuff = new float[allocLength];
-				}
-				memcpy(tbuff, pData, nFrames * sizeof(float));
-				hr = pEchoCaptureClient->ReleaseBuffer(numFramesAvailable);
-				if (FAILED(hr))
-					goto cleanup;
-				addEcho(tbuff, numFramesAvailable);
-			} else if (micPacketLength > 0) {
-				hr = pMicCaptureClient->GetBuffer(&pData, &numFramesAvailable, &flags, &devicePosition, &qpcPosition);
-				numFramesLeft = numFramesAvailable;
-				if (FAILED(hr))
-					goto cleanup;
 
 				UINT32 nFrames = numFramesAvailable * micpwfx->nChannels;
 				if (nFrames > allocLength) {
-					delete [] tbuff;
+					delete [] sbuff;
 					allocLength = nFrames;
-					tbuff = new float[allocLength];
+					sbuff = new short[allocLength];
 				}
-				memcpy(tbuff, pData, nFrames * sizeof(float));
+
+				memcpy(sbuff, pData, nFrames * sizeof(short));
 				hr = pMicCaptureClient->ReleaseBuffer(numFramesAvailable);
 				if (FAILED(hr))
 					goto cleanup;
-				addMic(tbuff, numFramesAvailable);
+				addMic(sbuff, numFramesAvailable);
 			}
+			if (! FAILED(hr))
+				WaitForSingleObject(hEvent, 100);
+		}
+	} else {
+		tbuff = new float[allocLength];
+		while (bRunning && ! FAILED(hr)) {
 			hr = pMicCaptureClient->GetNextPacketSize(&micPacketLength);
 			if (! FAILED(hr) && iEchoChannels)
 				hr = pEchoCaptureClient->GetNextPacketSize(&echoPacketLength);
+			if (FAILED(hr)) {
+				qWarning("WASAPIInput: GetNextPacketSize failed: hr=0x%08lx", hr);
+				goto cleanup;
+			}
+
+			while ((micPacketLength > 0) || (echoPacketLength > 0)) {
+				if (echoPacketLength > 0) {
+					hr = pEchoCaptureClient->GetBuffer(&pData, &numFramesAvailable, &flags, &devicePosition, &qpcPosition);
+					numFramesLeft = numFramesAvailable;
+					if (FAILED(hr)) {
+						qWarning("WASAPIInput: GetBuffer failed: hr=0x%08lx", hr);
+						goto cleanup;
+					}
+
+					UINT32 nFrames = numFramesAvailable * echopwfx->nChannels;
+					if (nFrames > allocLength) {
+						delete [] tbuff;
+						allocLength = nFrames;
+						tbuff = new float[allocLength];
+					}
+					memcpy(tbuff, pData, nFrames * sizeof(float));
+					hr = pEchoCaptureClient->ReleaseBuffer(numFramesAvailable);
+					if (FAILED(hr)) {
+						qWarning("WASAPIInput: ReleaseBuffer failed: hr=0x%08lx", hr);
+						goto cleanup;
+					}
+					addEcho(tbuff, numFramesAvailable);
+				} else if (micPacketLength > 0) {
+					hr = pMicCaptureClient->GetBuffer(&pData, &numFramesAvailable, &flags, &devicePosition, &qpcPosition);
+					numFramesLeft = numFramesAvailable;
+					if (FAILED(hr)) {
+						qWarning("WASAPIInput: GetBuffer failed: hr=0x%08lx", hr);
+						goto cleanup;
+					}
+
+					UINT32 nFrames = numFramesAvailable * micpwfx->nChannels;
+					if (nFrames > allocLength) {
+						delete [] tbuff;
+						allocLength = nFrames;
+						tbuff = new float[allocLength];
+					}
+					memcpy(tbuff, pData, nFrames * sizeof(float));
+					hr = pMicCaptureClient->ReleaseBuffer(numFramesAvailable);
+					if (FAILED(hr)) {
+						qWarning("WASAPIInput: ReleaseBuffer failed: hr=0x%08lx", hr);
+						goto cleanup;
+					}
+					addMic(tbuff, numFramesAvailable);
+				}
+				hr = pMicCaptureClient->GetNextPacketSize(&micPacketLength);
+				if (! FAILED(hr) && iEchoChannels)
+					hr = pEchoCaptureClient->GetNextPacketSize(&echoPacketLength);
+			}
+			if (! FAILED(hr))
+				WaitForSingleObject(hEvent, 2000);
 		}
-		if (! FAILED(hr))
-			WaitForSingleObject(hEvent, 2000);
 	}
 
 cleanup:
-	if (micpwfx)
+	if (micpwfx && !exclusive)
 		CoTaskMemFree(micpwfx);
 	if (echopwfx)
 		CoTaskMemFree(echopwfx);
@@ -509,21 +594,17 @@ cleanup:
 	if (pEchoDevice)
 		pEchoDevice->Release();
 
-	if (pEnumerator)
-		pEnumerator->Release();
-
 	if (hMmThread != NULL)
 		AvRevertMmThreadCharacteristics(hMmThread);
 
 	if (hEvent != NULL)
 		CloseHandle(hEvent);
 
-	if (tbuff)
-		delete [] tbuff;
+	delete [] tbuff;
+	delete [] sbuff;
 }
 
 WASAPIOutput::WASAPIOutput() {
-	bRunning = true;
 }
 
 WASAPIOutput::~WASAPIOutput() {
@@ -551,30 +632,16 @@ void WASAPIOutput::setVolumes(IMMDevice *pDevice, bool talking) {
 	int max = 0;
 	DWORD dwMumble = GetCurrentProcessId();
 
-	static int version = -1;
-	if (version == -1) {
-		OSVERSIONINFOEXW ovi;
-		memset(&ovi, 0, sizeof(ovi));
-
-		ovi.dwOSVersionInfoSize=sizeof(ovi);
-		GetVersionEx(reinterpret_cast<OSVERSIONINFOW *>(&ovi));
-
-		if ((ovi.dwMajorVersion <= 6) && (ovi.dwBuildNumber < 7100))
-			version = 0;
-		else
-			version = 1;
-	}
-
 	qmVolumes.clear();
 	if (qFuzzyCompare(g.s.fOtherVolume, 1.0f))
 		return;
 
 	// FIXME: Try to keep the session object around when returning volume.
 
-	if (SUCCEEDED(hr = pDevice->Activate(version ? __uuidof(IAudioSessionManager2) : __uuidof(IAudioSessionManager), CLSCTX_ALL, NULL, (void **) &pAudioSessionManager))) {
+	if (SUCCEEDED(hr = pDevice->Activate(bIsWin7 ? __uuidof(IAudioSessionManager2) : __uuidof(IAudioSessionManager), CLSCTX_ALL, NULL, (void **) &pAudioSessionManager))) {
 		IAudioSessionEnumerator *pEnumerator = NULL;
 		IAudioSessionQuery *pMysticQuery = NULL;
-		if (version == 0) {
+		if (! bIsWin7) {
 			if (SUCCEEDED(hr = pAudioSessionManager->QueryInterface(__uuidof(IAudioSessionQuery), (void **) &pMysticQuery))) {
 				hr = pMysticQuery->GetQueryInterface(&pEnumerator);
 			}
@@ -589,41 +656,7 @@ void WASAPIOutput::setVolumes(IMMDevice *pDevice, bool talking) {
 				for (int i=0;i<max;++i) {
 					IAudioSessionControl *pControl = NULL;
 					if (SUCCEEDED(hr = pEnumerator->GetSession(i, &pControl))) {
-						IAudioSessionControl2 *pControl2 = NULL;
-						if (SUCCEEDED(hr = pControl->QueryInterface(version ? __uuidof(IAudioSessionControl2) : __uuidof(IVistaAudioSessionControl2), (void **) &pControl2)))  {
-							DWORD pid;
-							if (SUCCEEDED(hr = pControl2->GetProcessId(&pid)) && (pid != dwMumble)) {
-								AudioSessionState ass;
-								if (SUCCEEDED(hr = pControl2->GetState(&ass)) && (ass != AudioSessionStateExpired)) {
-									GUID group;
-									if (SUCCEEDED(hr = pControl2->GetGroupingParam(&group))) {
-										QUuid quuid(group);
-										if (! seen.contains(quuid)) {
-											seen.insert(quuid);
-											ISimpleAudioVolume *pVolume = NULL;
-											if (SUCCEEDED(hr = pControl2->QueryInterface(__uuidof(ISimpleAudioVolume), (void **) &pVolume))) {
-												BOOL bMute = TRUE;
-												bool keep = false;
-												if (SUCCEEDED(hr = pVolume->GetMute(&bMute)) && ! bMute) {
-													float fVolume = 1.0f;
-													if (SUCCEEDED(hr = pVolume->GetMasterVolume(&fVolume)) && ! qFuzzyCompare(fVolume,0.0f)) {
-														float fSetVolume = fVolume * g.s.fOtherVolume;
-														if (SUCCEEDED(hr = pVolume->SetMasterVolume(fSetVolume, NULL))) {
-															hr = pVolume->GetMasterVolume(&fSetVolume);
-															qmVolumes.insert(pVolume, VolumePair(fVolume,fSetVolume));
-															keep = true;
-														}
-													}
-												}
-												if (! keep)
-													pVolume->Release();
-											}
-										}
-									}
-								}
-							}
-							pControl2->Release();
-						}
+						setVolumeForSessionControl(pControl, dwMumble, seen);
 						pControl->Release();
 					}
 				}
@@ -636,19 +669,123 @@ void WASAPIOutput::setVolumes(IMMDevice *pDevice, bool talking) {
 	}
 }
 
+bool WASAPIOutput::setVolumeForSessionControl2(IAudioSessionControl2 *control2, const DWORD mumblePID, QSet<QUuid> &seen) {
+	HRESULT hr;
+	DWORD pid;
+	
+	// Don't set the volume for our own control
+	if (FAILED(hr = control2->GetProcessId(&pid)) || (pid == mumblePID))
+		return true;
+	
+	// Don't work on expired audio sessions
+	AudioSessionState ass;
+	if (FAILED(hr = control2->GetState(&ass)) || (ass == AudioSessionStateExpired))
+		return false;
+	
+	// Don't act twice on the same session
+	GUID group;
+	if (FAILED(hr = control2->GetGroupingParam(&group)))
+		return false;
+	
+	QUuid quuid(group);
+	if (seen.contains(quuid))
+		return true;
+	
+	seen.insert(quuid);
+	
+	// Adjust volume
+	ISimpleAudioVolume *pVolume = NULL;
+	if (FAILED(hr = control2->QueryInterface(__uuidof(ISimpleAudioVolume), (void **) &pVolume)))
+		return false;
+	
+	BOOL bMute = TRUE;
+	bool keep = false;
+	if (SUCCEEDED(hr = pVolume->GetMute(&bMute)) && ! bMute) {
+		float fVolume = 1.0f;
+		if (SUCCEEDED(hr = pVolume->GetMasterVolume(&fVolume)) && ! qFuzzyCompare(fVolume,0.0f)) {
+			float fSetVolume = fVolume * g.s.fOtherVolume;
+			if (SUCCEEDED(hr = pVolume->SetMasterVolume(fSetVolume, NULL))) {
+				hr = pVolume->GetMasterVolume(&fSetVolume);
+				qmVolumes.insert(pVolume, VolumePair(fVolume,fSetVolume));
+				keep = true;
+			}
+		}
+	}
+	
+	if (! keep)
+		pVolume->Release();
+	
+	return true;
+}
+
+bool WASAPIOutput::setVolumeForSessionControl(IAudioSessionControl *control, const DWORD mumblePID, QSet<QUuid> &seen) {
+	HRESULT hr;
+	IAudioSessionControl2 *pControl2 = NULL;
+
+	if (!SUCCEEDED(hr = control->QueryInterface(bIsWin7 ? __uuidof(IAudioSessionControl2) : __uuidof(IVistaAudioSessionControl2), (void **) &pControl2)))
+		return false;
+	
+	bool result = setVolumeForSessionControl2(pControl2, mumblePID, seen);
+	
+	pControl2->Release();
+	return result;
+}
+
+static void SetDuckingOptOut(IMMDevice *pDevice) {
+	if (!bIsWin7)
+		return;
+
+	HRESULT hr;
+	IAudioSessionManager2 *pSessionManager2 = NULL;
+	IAudioSessionControl *pSessionControl = NULL;
+	IAudioSessionControl2 *pSessionControl2 = NULL;
+
+	// Get session manager & control1+2 to disable ducking
+	hr = pDevice->Activate(__uuidof(IAudioSessionManager2), CLSCTX_ALL, NULL, reinterpret_cast<void**>(&pSessionManager2));
+	if (FAILED(hr)) {
+		qWarning("WASAPIOutput: Activate AudioSessionManager2 failed: hr=0x%08lx", hr);
+		goto cleanup;
+	}
+
+	hr = pSessionManager2->GetAudioSessionControl(NULL, 0, &pSessionControl);
+	if (FAILED(hr)) {
+		qWarning("WASAPIOutput: GetAudioSessionControl failed: hr=0x%08lx", hr);
+		goto cleanup;
+	}
+
+	hr = pSessionControl->QueryInterface(__uuidof(IAudioSessionControl2), reinterpret_cast<void**>(&pSessionControl2));
+	if (FAILED(hr)) {
+		qWarning("WASAPIOutput: Querying SessionControl2 failed: hr=0x%08lx", hr);
+		goto cleanup;
+	}
+
+	hr = pSessionControl2->SetDuckingPreference(TRUE);
+	if (FAILED(hr)) {
+		qWarning("WASAPIOutput: Failed to set ducking preference: hr=0x%08lx", hr);
+		goto cleanup;
+	}
+
+cleanup:
+	if (pSessionControl2)
+		pSessionControl2->Release();
+
+	if (pSessionControl)
+		pSessionControl->Release();
+
+	if (pSessionManager2)
+		pSessionManager2->Release();
+}
+
 void WASAPIOutput::run() {
 	HRESULT hr;
-	IMMDeviceEnumerator *pEnumerator = NULL;
 	IMMDevice *pDevice = NULL;
 	IAudioClient *pAudioClient = NULL;
 	IAudioRenderClient *pRenderClient = NULL;
 	WAVEFORMATEX *pwfx = NULL;
 	WAVEFORMATEXTENSIBLE *pwfxe = NULL;
 	UINT32 bufferFrameCount;
-	REFERENCE_TIME def, min, latency;
+	REFERENCE_TIME def, min, latency, want;
 	UINT32 numFramesAvailable;
-	UINT32 packetLength = 0;
-	UINT32 wantLength;
 	HANDLE hEvent;
 	BYTE *pData;
 	DWORD dwTaskIndex = 0;
@@ -658,6 +795,7 @@ void WASAPIOutput::run() {
 	QMap<DWORD, float> qmVolumes;
 	bool lastspoke = false;
 	REFERENCE_TIME bufferDuration = (g.s.iOutputDelay > 1) ? (g.s.iOutputDelay + 1) * 100000 : 0;
+	bool exclusive = false;
 
 	CoInitialize(NULL);
 
@@ -668,97 +806,128 @@ void WASAPIOutput::run() {
 		qWarning("WASAPIOutput: Failed to set Pro Audio thread priority");
 	}
 
-	hr = CoCreateInstance(__uuidof(MMDeviceEnumerator), NULL, CLSCTX_ALL, __uuidof(IMMDeviceEnumerator), reinterpret_cast<void **>(&pEnumerator));
+	// Open the output device.
+	pDevice = openNamedOrDefaultDevice(g.s.qsWASAPIOutput, eRender, eCommunications);
+	if (!pDevice)
+		goto cleanup;
 
-	if (! pEnumerator || FAILED(hr)) {
-		qWarning("WASAPIOutput: Failed to instatiate enumerator");
-		return;
-	}
-
-	if (! g.s.qsWASAPIOutput.isEmpty()) {
-		STACKVAR(wchar_t, devname, g.s.qsWASAPIOutput.length() + 1);
-		int len = g.s.qsWASAPIOutput.toWCharArray(devname);
-		devname[len] = 0;
-		hr = pEnumerator->GetDevice(devname, &pDevice);
-		if (FAILED(hr)) {
-			qWarning("WASAPIOutput: Failed to open selected input device, falling back to default");
-		}
-	}
-
-	if (! pDevice) {
-		hr = pEnumerator->GetDefaultAudioEndpoint(eRender, eCommunications, &pDevice);
-		if (FAILED(hr)) {
-			qWarning("WASAPIOutput: Failed to open output device");
-			goto cleanup;
-		}
-		wchar_t *devname = NULL;
-		hr = pDevice->GetId(&devname);
-		if (FAILED(hr)) {
-			qWarning("WASAPIOutput: Failed to query output device");
-			goto cleanup;
-		}
-		pDevice->Release();
-		hr = pEnumerator->GetDevice(devname, &pDevice);
-		if (FAILED(hr)) {
-			qWarning("WASAPIOutput: Failed to reopen default output device");
-			goto cleanup;
-		}
-		CoTaskMemFree(devname);
-	}
-
-	pEnumerator->Release();
-	pEnumerator = NULL;
+	// Opt-out of the Windows 7 ducking behavior
+	SetDuckingOptOut(pDevice);
 
 	hr = pDevice->Activate(__uuidof(IAudioClient), CLSCTX_ALL, NULL, (void **) &pAudioClient);
 	if (FAILED(hr)) {
-		qWarning("WASAPIOutput: Activate AudioClient failed");
-		goto cleanup;
-	}
-
-	hr = pAudioClient->GetMixFormat(&pwfx);
-	if (FAILED(hr)) {
-		qWarning("WASAPIOutput: GetMixFormat failed: %llx", hr);
-		goto cleanup;
-	}
-	pwfxe = reinterpret_cast<WAVEFORMATEXTENSIBLE *>(pwfx);
-
-	if (FAILED(hr) || (pwfx->wBitsPerSample != (sizeof(float) * 8)) || (pwfxe->SubFormat != KSDATAFORMAT_SUBTYPE_IEEE_FLOAT)) {
-		qWarning("WASAPIOutput: Subformat is not IEEE Float");
-		goto cleanup;
-	}
-
-	iMixerFreq = pwfx->nSamplesPerSec;
-
-	hr = pAudioClient->Initialize(AUDCLNT_SHAREMODE_SHARED, AUDCLNT_STREAMFLAGS_EVENTCALLBACK, bufferDuration, 0, pwfx, NULL);
-	if (FAILED(hr)) {
-		qWarning("WASAPIOutput: Initialize failed");
+		qWarning("WASAPIOutput: Activate AudioClient failed: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 
 	pAudioClient->GetDevicePeriod(&def, &min);
-	pAudioClient->GetBufferSize(&bufferFrameCount);
+	want = qMax<REFERENCE_TIME>(min, 100000);
+	qWarning("WASAPIOutput: Latencies %lld %lld => %lld", def, min, want);
+
+	if (g.s.bExclusiveOutput) {
+		hr = pAudioClient->GetMixFormat(&pwfx);
+		if (FAILED(hr)) {
+			qWarning("WASAPIOutput: GetMixFormat failed: hr=0x%08lx", hr);
+			goto cleanup;
+		}
+
+		pwfxe = reinterpret_cast<WAVEFORMATEXTENSIBLE *>(pwfx);
+
+		if (!g.s.bPositionalAudio) {
+			pwfx->nChannels = 2;
+			pwfxe->dwChannelMask = KSAUDIO_SPEAKER_STEREO;
+		}
+
+		pwfx->cbSize = 0;
+		pwfx->wFormatTag = WAVE_FORMAT_PCM;
+		pwfx->nSamplesPerSec = 48000;
+		pwfx->wBitsPerSample = 16;
+		pwfx->nBlockAlign = pwfx->nChannels * pwfx->wBitsPerSample / 8;
+		pwfx->nAvgBytesPerSec = pwfx->nBlockAlign * pwfx->nSamplesPerSec;
+
+		hr = pAudioClient->Initialize(AUDCLNT_SHAREMODE_EXCLUSIVE, AUDCLNT_STREAMFLAGS_EVENTCALLBACK, want, want, pwfx, NULL);
+		if (SUCCEEDED(hr)) {
+			eSampleFormat = SampleShort;
+			exclusive = true;
+			qWarning("WASAPIOutput: Successfully opened exclusive mode");
+		} else {
+			CoTaskMemFree(pwfx);
+
+			pwfxe = NULL;
+			pwfx = NULL;
+		}
+	}
+
+	if (!  pwfxe) {
+		if (g.s.bExclusiveOutput)
+			qWarning("WASAPIOutput: Failed to open exclusive mode.");
+
+		hr = pAudioClient->GetMixFormat(&pwfx);
+		if (FAILED(hr)) {
+			qWarning("WASAPIOutput: GetMixFormat failed: hr=0x%08lx", hr);
+			goto cleanup;
+		}
+		pwfxe = reinterpret_cast<WAVEFORMATEXTENSIBLE *>(pwfx);
+
+		if (FAILED(hr) || (pwfx->wBitsPerSample != (sizeof(float) * 8)) || (pwfxe->SubFormat != KSDATAFORMAT_SUBTYPE_IEEE_FLOAT)) {
+			qWarning("WASAPIOutput: Subformat is not IEEE Float: hr=0x%08lx", hr);
+			goto cleanup;
+		}
+
+		if (!g.s.bPositionalAudio) {
+			pwfxe->Format.nChannels = 2;
+			pwfxe->Format.nBlockAlign = pwfx->nChannels * pwfx->wBitsPerSample / 8;
+			pwfxe->Format.nAvgBytesPerSec = pwfx->nBlockAlign * pwfx->nSamplesPerSec;
+			pwfxe->dwChannelMask = KSAUDIO_SPEAKER_STEREO;
+
+			WAVEFORMATEX *closestFormat = NULL;
+			hr = pAudioClient->IsFormatSupported(AUDCLNT_SHAREMODE_SHARED, pwfx, &closestFormat);
+			if (hr == S_FALSE) {
+				qWarning("WASAPIOutput: Driver says no to 2 channel output. Closest format: %d channels @ %d kHz", closestFormat->nChannels, closestFormat->nSamplesPerSec);
+				CoTaskMemFree(pwfx);
+
+				pwfx = NULL;
+				pAudioClient->GetMixFormat(&pwfx);
+				pwfxe = reinterpret_cast<WAVEFORMATEXTENSIBLE *>(pwfx);
+			} else if (FAILED(hr)) {
+				qWarning("WASAPIOutput: IsFormatSupported failed: hr=0x%08lx", hr);
+			}
+
+			if (closestFormat)
+				CoTaskMemFree(closestFormat);
+		}
+
+		hr = pAudioClient->Initialize(AUDCLNT_SHAREMODE_SHARED, AUDCLNT_STREAMFLAGS_EVENTCALLBACK, bufferDuration, 0, pwfx, NULL);
+		if (FAILED(hr)) {
+			qWarning("WASAPIOutput: Initialize failed: hr=0x%08lx", hr);
+			goto cleanup;
+		}
+	}
+
 	pAudioClient->GetStreamLatency(&latency);
+	pAudioClient->GetBufferSize(&bufferFrameCount);
+	qWarning("WASAPIOutput: Stream Latency %lld (%d)", latency, bufferFrameCount);
+
+	iMixerFreq = pwfx->nSamplesPerSec;
 
 	qWarning("WASAPIOutput: Periods %lldus %lldus (latency %lldus)", def / 10LL, min / 10LL, latency / 10LL);
 	qWarning("WASAPIOutput: Buffer is %dus (%d)", (bufferFrameCount * 1000000) / iMixerFreq, g.s.iOutputDelay);
 
 	hr = pAudioClient->GetService(__uuidof(IAudioRenderClient), (void**)&pRenderClient);
 	if (FAILED(hr)) {
-		qWarning("WASAPIOutput: GetService failed");
+		qWarning("WASAPIOutput: GetService failed: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 
-	wantLength = lroundf(ceilf((iFrameSize * pwfx->nSamplesPerSec) / (SAMPLE_RATE * 1.0f)));
-
 	pAudioClient->SetEventHandle(hEvent);
 	if (FAILED(hr)) {
-		qWarning("WASAPIOutput: Failed to set event");
+		qWarning("WASAPIOutput: Failed to set event: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 
 	hr = pAudioClient->Start();
 	if (FAILED(hr)) {
-		qWarning("WASAPIOutput: Failed to start");
+		qWarning("WASAPIOutput: Failed to start: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 
@@ -775,30 +944,45 @@ void WASAPIOutput::run() {
 	initializeMixer(chanmasks);
 
 	bool mixed = false;
+	numFramesAvailable = 0;
+
 	while (bRunning && ! FAILED(hr)) {
-		hr = pAudioClient->GetCurrentPadding(&numFramesAvailable);
-		if (FAILED(hr))
-			goto cleanup;
+		if (!exclusive) {
+			// Attenuate stream volumes.
+			if (lastspoke != (g.bAttenuateOthers || mixed)) {
+				lastspoke = g.bAttenuateOthers || mixed;
+				setVolumes(pDevice, lastspoke);
+			}
 
-		packetLength = bufferFrameCount - numFramesAvailable;
-
-		if (lastspoke != (g.bAttenuateOthers || mixed)) {
-			lastspoke = g.bAttenuateOthers || mixed;
-			setVolumes(pDevice, lastspoke);
+			hr = pAudioClient->GetCurrentPadding(&numFramesAvailable);
+			if (FAILED(hr)) {
+				qWarning("WASAPIOutput: GetCurrentPadding failed: hr=0x%08lx", hr);
+				goto cleanup;
+			}
 		}
 
+		UINT32 packetLength = bufferFrameCount - numFramesAvailable;
+
 		while (packetLength > 0) {
 			hr = pRenderClient->GetBuffer(packetLength, &pData);
-			if (FAILED(hr))
+			if (FAILED(hr)) {
+				qWarning("WASAPIOutput: GetBuffer failed: hr=0x%08lx", hr);
 				goto cleanup;
+			}
 
 			mixed = mix(reinterpret_cast<float *>(pData), packetLength);
 			if (mixed)
 				hr = pRenderClient->ReleaseBuffer(packetLength, 0);
 			else
 				hr = pRenderClient->ReleaseBuffer(packetLength, AUDCLNT_BUFFERFLAGS_SILENT);
-			if (FAILED(hr))
+			if (FAILED(hr)) {
+				qWarning("WASAPIOutput: ReleaseBuffer failed: hr=0x%08lx", hr);
 				goto cleanup;
+			}
+
+			// Exclusive mode rendering ends here.
+			if (exclusive)
+				break;
 
 			if (!g.s.bAttenuateOthers && !g.bAttenuateOthers) {
 				mixed = false;
@@ -810,13 +994,15 @@ void WASAPIOutput::run() {
 			}
 
 			hr = pAudioClient->GetCurrentPadding(&numFramesAvailable);
-			if (FAILED(hr))
+			if (FAILED(hr)) {
+				qWarning("WASAPIOutput: GetCurrentPadding failed: hr=0x%08lx", hr);
 				goto cleanup;
+			}
 
 			packetLength = bufferFrameCount - numFramesAvailable;
 		}
 		if (! FAILED(hr))
-			WaitForSingleObject(hEvent, 2000);
+			WaitForSingleObject(hEvent, exclusive ? 100 : 2000);
 	}
 
 cleanup:
@@ -834,9 +1020,6 @@ cleanup:
 	if (pDevice)
 		pDevice->Release();
 
-	if (pEnumerator)
-		pEnumerator->Release();
-
 	if (hMmThread != NULL)
 		AvRevertMmThreadCharacteristics(hMmThread);
 
