@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,16 +28,19 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "murmur_pch.h"
+
 #include "ServerDB.h"
-#include "User.h"
-#include "Channel.h"
-#include "Group.h"
+
 #include "ACL.h"
-#include "Server.h"
-#include "ServerUser.h"
-#include "Meta.h"
+#include "Channel.h"
 #include "Connection.h"
 #include "DBus.h"
+#include "Group.h"
+#include "Meta.h"
+#include "Server.h"
+#include "ServerUser.h"
+#include "User.h"
 
 #define SQLDO(x) ServerDB::exec(query, QLatin1String(x), true)
 #define SQLMAY(x) ServerDB::exec(query, QLatin1String(x), false, false)
@@ -110,9 +113,9 @@ ServerDB::ServerDB() {
 		}
 		if (found) {
 			QFileInfo fi(db->databaseName());
-			qWarning("ServerDB: Openend SQLite database %s", qPrintable(fi.absoluteFilePath()));
+			qWarning("ServerDB: Opened SQLite database %s", qPrintable(fi.absoluteFilePath()));
 			if (! fi.isWritable())
-				qFatal("ServerDB: Database is not writeable");
+				qFatal("ServerDB: Database is not writable");
 		}
 	} else {
 		db->setDatabaseName(Meta::mp.qsDatabase);
@@ -568,6 +571,16 @@ void Server::initialize() {
 		query.addBindValue(iServerNum);
 		query.addBindValue(QLatin1String("SuperUser"));
 		SQLEXEC();
+
+		int length = qrand() % 8 + 8;
+		QString pw;
+		pw.reserve(length);
+
+		while (length--)
+			pw.append(QChar(qrand() % 94 + 33));
+
+		ServerDB::setSUPW(iServerNum, pw);
+		log(QString("Password for 'SuperUser' set to '%2'").arg(pw));
 	}
 
 	SQLPREP("SELECT COUNT(*) FROM `%1acl` WHERE `server_id`=?");
@@ -789,10 +802,10 @@ QMap<int, QString> Server::getRegistration(int id) {
 // -1 Wrong PW
 // -2 Anonymous
 
-int Server::authenticate(QString &name, const QString &pw, const QStringList &emails, const QString &certhash, bool bStrongCert, const QList<QSslCertificate> &certs) {
+int Server::authenticate(QString &name, const QString &pw, int sessionId, const QStringList &emails, const QString &certhash, bool bStrongCert, const QList<QSslCertificate> &certs) {
 	int res = -2;
 
-	emit authenticateSig(res, name, certs, certhash, bStrongCert, pw);
+	emit authenticateSig(res, name, sessionId, certs, certhash, bStrongCert, pw);
 
 	if (res != -2) {
 		// External authentication handled it. Ignore certificate completely.
@@ -1131,24 +1144,16 @@ void Server::removeLink(Channel *c, Channel *l) {
 
 	QSqlQuery &query = *th.qsqQuery;
 
-	if (l) {
-		SQLPREP("DELETE FROM `%1channel_links` WHERE `server_id` = ? AND `channel_id` = ? AND `link_id` = ?");
-		query.addBindValue(iServerNum);
-		query.addBindValue(c->iId);
-		query.addBindValue(l->iId);
-		SQLEXEC();
+	SQLPREP("DELETE FROM `%1channel_links` WHERE `server_id` = ? AND `channel_id` = ? AND `link_id` = ?");
+	query.addBindValue(iServerNum);
+	query.addBindValue(c->iId);
+	query.addBindValue(l->iId);
+	SQLEXEC();
 
-		query.addBindValue(iServerNum);
-		query.addBindValue(l->iId);
-		query.addBindValue(c->iId);
-		SQLEXEC();
-	} else {
-		SQLPREP("DELETE FROM `%1channel_links` WHERE `server_id` = ? AND (`channel_id` = ? OR `link_id` = ?)");
-		query.addBindValue(iServerNum);
-		query.addBindValue(c->iId);
-		query.addBindValue(c->iId);
-		SQLEXEC();
-	}
+	query.addBindValue(iServerNum);
+	query.addBindValue(l->iId);
+	query.addBindValue(c->iId);
+	SQLEXEC();
 }
 
 Channel *Server::addChannel(Channel *p, const QString &name, bool temporary, int position) {
@@ -1174,14 +1179,21 @@ Channel *Server::addChannel(Channel *p, const QString &name, bool temporary, int
 		query.addBindValue(id);
 		query.addBindValue(name);
 		SQLEXEC();
-	}
 
-	// Add channel sorting information
-	SQLPREP("INSERT INTO `%1channel_info` ( `server_id`, `channel_id`, `key`, `value`) VALUES(?,?,?,?)");
-	query.addBindValue(iServerNum);
-	query.addBindValue(id);
-	query.addBindValue(ServerDB::Channel_Position);
-	query.addBindValue(QVariant(position).toString());
+		// Delete old channel_info rows
+		SQLPREP("DELETE FROM `%1channel_info` WHERE `server_id` = ? AND `channel_id` = ?");
+		query.addBindValue(iServerNum);
+		query.addBindValue(id);
+		SQLEXEC();
+
+		// Add channel sorting information
+		SQLPREP("INSERT INTO `%1channel_info` ( `server_id`, `channel_id`, `key`, `value`) VALUES(?,?,?,?)");
+		query.addBindValue(iServerNum);
+		query.addBindValue(id);
+		query.addBindValue(ServerDB::Channel_Position);
+		query.addBindValue(QVariant(position).toString());
+		SQLEXEC();
+	}
 
 	Channel *c = new Channel(id, name, p);
 	c->bTemporary = temporary;
@@ -1570,7 +1582,7 @@ void Server::setConf(const QString &key, const QVariant &value) {
 	ServerDB::setConf(iServerNum, key, value);
 }
 
-void Server::dblog(const QString &str) {
+void Server::dblog(const QString &str) const {
 	TransactionHolder th;
 	QSqlQuery &query = *th.qsqQuery;
 
@@ -1598,6 +1610,13 @@ void Server::dblog(const QString &str) {
 	SQLEXEC();
 }
 
+void ServerDB::wipeLogs() {
+	TransactionHolder th;
+	QSqlQuery &query = *th.qsqQuery;
+
+	SQLDO("DELETE FROM %1slog");
+}
+
 QList<QPair<unsigned int, QString> > ServerDB::getLog(int server_id, unsigned int offs_min, unsigned int offs_max) {
 	TransactionHolder th;
 	QSqlQuery &query = *th.qsqQuery;
