@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -85,7 +85,7 @@ static void murmurMessageOutput(QtMsgType type, const char *msg) {
 	if (! qfLog || ! qfLog->isOpen()) {
 #ifdef Q_OS_UNIX
 		if (! detach)
-			fprintf(stderr, "%s\n", msg);
+			fprintf(stderr, "%s\n", qPrintable(m));
 		else
 			qlErrors << m;
 #else
@@ -140,8 +140,9 @@ int main(int argc, char **argv) {
 		::MessageBoxA(NULL, "Mumble requires a SSE capable processor (Pentium 3 / Ahtlon-XP)", "Mumble", MB_OK | MB_ICONERROR);
 		exit(0);
 	}
-#endif
 
+	SetDllDirectory(L"");
+#endif
 	int res;
 
 #ifdef USE_ICE
@@ -193,11 +194,13 @@ int main(int argc, char **argv) {
 	QString inifile;
 	QString supw;
 	bool wipeSsl = false;
+	bool wipeLogs = false;
 	int sunum = 1;
-#ifndef Q_OS_WIN
+#ifdef Q_OS_UNIX
 	bool readPw = false;
 #endif
 
+	qsrand(QDateTime::currentDateTime().toTime_t());
 	qInstallMsgHandler(murmurMessageOutput);
 
 #ifdef Q_OS_WIN
@@ -207,15 +210,23 @@ int main(int argc, char **argv) {
 	for (int i=1;i<argc;i++) {
 		bool bLast = false;
 		QString arg = QString(argv[i]).toLower();
-		if ((arg == "-supw") && (i+1 < argc)) {
-			i++;
-			supw = argv[i];
+		if ((arg == "-supw")) {
 			detach = false;
 			if (i+1 < argc) {
 				i++;
-				sunum = QString::fromLatin1(argv[i]).toInt();
+				supw = argv[i];
+				if (i+1 < argc) {
+					i++;
+					sunum = QString::fromLatin1(argv[i]).toInt();
+				}
+				bLast = true;
+			} else {
+#ifdef Q_OS_UNIX
+				qFatal("-supw expects the password on the command line - maybe you meant -readsupw?");
+#else
+				qFatal("-supw expects the password on the command line");
+#endif
 			}
-			bLast = true;
 #ifdef Q_OS_UNIX
 		} else if ((arg == "-readsupw")) {
 			detach = false;
@@ -231,6 +242,8 @@ int main(int argc, char **argv) {
 			inifile=argv[i];
 		} else if ((arg == "-wipessl")) {
 			wipeSsl = true;
+		} else if ((arg == "-wipelogs")) {
+			wipeLogs = true;
 		} else if ((arg == "-fg")) {
 			detach = false;
 		} else if ((arg == "-v")) {
@@ -247,8 +260,13 @@ int main(int argc, char **argv) {
 			       "  -readsupw [srv]  Reads password for server srv from standard input.\n"
 #endif
 			       "  -v               Add verbose output.\n"
-			       "  -fg              Don't detach from console [Unix-like systems only].\n"
+#ifdef Q_OS_UNIX
+			       "  -fg              Don't detach from console.\n"
+#else
+			       "  -fg              Don't write to the log file.\n"
+#endif
 			       "  -wipessl         Remove SSL certificates from database.\n"
+			       "  -wipelogs        Remove all log entries from database.\n"
 			       "  -version         Show version information.\n"
 			       "If no inifile is provided, murmur will search for one in \n"
 			       "default locations.", argv[0]);
@@ -269,13 +287,44 @@ int main(int argc, char **argv) {
 		}
 	}
 
+#ifdef Q_OS_UNIX
+	inifile = unixhandler.trySystemIniFiles(inifile);
+#endif
 
 	Meta::mp.read(inifile);
-	MumbleSSL::addSystemCA();
+
+	// need to open log file early so log dir can be root owned:
+	// http://article.gmane.org/gmane.comp.security.oss.general/4404
+	if (detach && ! Meta::mp.qsLogfile.isEmpty()) {
+		qfLog = new QFile(Meta::mp.qsLogfile);
+		if (! qfLog->open(QIODevice::WriteOnly | QIODevice::Append | QIODevice::Text)) {
+			delete qfLog;
+			qfLog = NULL;
+#ifdef Q_OS_UNIX
+			fprintf(stderr, "murmurd: failed to open logfile %s: no logging will be done\n", qPrintable(Meta::mp.qsLogfile));
+#else
+			qWarning("Failed to open logfile %s. No logging will be performed.", qPrintable(Meta::mp.qsLogfile));
+#endif
+		} else {
+			qfLog->setTextModeEnabled(true);
+			QFileInfo qfi(*qfLog);
+			Meta::mp.qsLogfile = qfi.absoluteFilePath();
+#ifdef Q_OS_UNIX
+			if (Meta::mp.uiUid != 0 && fchown(qfLog->handle(), Meta::mp.uiUid, Meta::mp.uiGid) == -1) {
+				qFatal("can't change log file owner to %d %d:%d - %s", qfLog->handle(), Meta::mp.uiUid, Meta::mp.uiGid, strerror(errno));
+			}
+#endif
+		}
+	} else {
+		detach = false;
+	}
 
 #ifdef Q_OS_UNIX
 	unixhandler.setuid();
 #endif
+
+	MumbleSSL::addSystemCA();
+
 	ServerDB db;
 
 	meta = new Meta();
@@ -287,7 +336,8 @@ int main(int argc, char **argv) {
 
 		printf("Password: ");
 		fflush(NULL);
-		fgets(password, 255, stdin);
+		if (fgets(password, 255, stdin) != password)
+			qFatal("No password provided");
 		p = strchr(password, '\r');
 		if (p)
 			*p = 0;
@@ -315,25 +365,11 @@ int main(int argc, char **argv) {
 		}
 	}
 
-	if (detach && ! Meta::mp.qsLogfile.isEmpty()) {
-		qfLog = new QFile(Meta::mp.qsLogfile);
-		if (! qfLog->open(QIODevice::WriteOnly | QIODevice::Append | QIODevice::Text)) {
-			delete qfLog;
-			qfLog = NULL;
-#ifdef Q_OS_UNIX
-			fprintf(stderr, "murmurd: failed to open logfile %s: no logging will be done\n",qPrintable(Meta::mp.qsLogfile));
-#else
-			qWarning("Failed to open logfile %s. Will not detach.",qPrintable(Meta::mp.qsLogfile));
-			detach = false;
-#endif
-		} else {
-			qfLog->setTextModeEnabled(true);
-			QFileInfo qfi(*qfLog);
-			Meta::mp.qsLogfile = qfi.absoluteFilePath();
-		}
-	} else {
-		detach = false;
+	if (wipeLogs) {
+		qWarning("Removing all log entries from the database.");
+		ServerDB::wipeLogs();
 	}
+
 #ifdef Q_OS_UNIX
 	if (detach) {
 		if (fork() != 0) {
@@ -356,7 +392,8 @@ int main(int argc, char **argv) {
 			}
 		}
 
-		chdir("/");
+		if (chdir("/") != 0)
+			fprintf(stderr, "Failed to chdir to /");
 		int fd;
 
 		fd = open("/dev/null", O_RDONLY);
@@ -435,10 +472,8 @@ int main(int argc, char **argv) {
 	IceStop();
 #endif
 
-	if (qfLog) {
-		delete qfLog;
-		qfLog = NULL;
-	}
+	delete qfLog;
+	qfLog = NULL;
 
 	delete meta;
 
